@@ -11,6 +11,7 @@
 #include "framedata.h"
 #include "link.h"
 #include "object.h"
+#include "tag.h"
 #include "tilingresult.h"
 #include "x11-types.h"
 
@@ -122,6 +123,7 @@ public:
     void setLayout(LayoutAlgorithm l) { layout = l; }
     int getSelection() { return selection; }
     size_t clientCount() { return clients.size(); }
+    int clientIndex(Client* client);
     std::shared_ptr<Frame> neighbour(Direction direction);
     std::vector<Client*> removeAllClients();
 
@@ -130,7 +132,7 @@ public:
 
     friend class Frame;
     void setVisible(bool visible);
-    int getInnerNeighbourIndex(Direction direction);
+    int getInnerNeighbourIndex(Direction direction, DirectionLevel depth, int startIndex = -1);
     DynAttribute_<int> client_count_;
     DynAttribute_<int> selectionAttr_;
     DynAttribute_<LayoutAlgorithm> algorithmAttr_;
