@@ -7,9 +7,11 @@
 #include <csignal>
 #include <cstdio>
 #include <cstdlib>
+#include <cstring>
 #include <iostream>
 #include <vector>
 
+#include "autostart.h"
 #include "client.h"
 #include "clientmanager.h"
 #include "command.h"
@@ -45,6 +47,7 @@ using std::pair;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
+using std::vector;
 
 // globals:
 int g_verbose = 0;
@@ -52,23 +55,21 @@ Display*    g_display;
 Window      g_root;
 
 // module internals:
-static char*    g_autostart_path = nullptr; // if not set, then find it in $HOME or $XDG_CONFIG_HOME
 static bool     g_exec_before_quit = false;
-static char**   g_exec_args = nullptr;
+static vector<string> g_exec_args = {};
 static XMainLoop* g_main_loop = nullptr;
 
 int quit();
 int version(Output output);
-void execute_autostart_file();
-int spawn(int argc, char** argv);
-int wmexec(int argc, char** argv);
-static void remove_zombies(int signal);
+int spawn(Input input, Output output);
+int wmexec(Input input);
 int custom_hook_emit(Input input);
 
 unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
     MetaCommands* meta_commands = root->meta_commands.get();
     GlobalCommands* global_cmds = root->global_commands.get();
 
+    Autostart* autostart = root->autostart();
     ClientManager* clients = root->clients();
     KeyManager *keys = root->keys();
     MonitorManager* monitors = root->monitors();
@@ -90,7 +91,7 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
                                            &MetaCommands::completeCommandShifted1}},
         {"silent",         {meta_commands, &MetaCommands::silentCommand,
                                            &MetaCommands::completeCommandShifted1}},
-        {"reload",         {[] { execute_autostart_file(); return 0; }}},
+        {"reload",         {autostart, &Autostart::reloadCmdDummyOutput }},
         {"version",        { version }},
         {"list_commands",  { list_commands }},
         {"list_monitors",  {monitors, &MonitorManager::list_monitors }},
@@ -204,8 +205,7 @@ unique_ptr<CommandTable> commands(shared_ptr<Root> root) {
                                             &MetaCommands::print_object_tree_complete} },
         {"substitute",     { meta_commands, &MetaCommands::substitute_cmd,
                                             &MetaCommands::substitute_complete} },
-        {"foreach",        { meta_commands, &MetaCommands::foreachCmd,
-                                            &MetaCommands::foreachComplete} },
+        {"foreach",        { meta_commands, &MetaCommands::foreachCommand} },
         {"sprintf",        { meta_commands, &MetaCommands::sprintf_cmd,
                                             &MetaCommands::sprintf_complete} },
         {"new_attr",       { meta_commands, &MetaCommands::new_attr_cmd,
@@ -263,98 +263,25 @@ int custom_hook_emit(Input input) {
     return 0;
 }
 
-static void execvp_helper(char *const command[]) {
-    execvp(command[0], command);
-    std::cerr << "herbstluftwm: execvp \"" << command[0] << "\"";
-    perror(" failed");
-}
-
-// spawn() heavily inspired by dwm.c
-int spawn(int argc, char** argv) {
-    if (argc < 2) {
+int spawn(Input input, Output output) {
+    if (input.empty()) {
         return HERBST_NEED_MORE_ARGS;
     }
-    if (fork() == 0) {
-        // only look in child
-        if (g_display) {
-            close(ConnectionNumber(g_display));
-        }
-        // shift all args in argv by 1 to the front
-        // so that we have space for a NULL entry at the end for execvp
-        char** execargs = argv_duplicate(argc, argv);
-        free(execargs[0]);
-        int i;
-        for (i = 0; i < argc-1; i++) {
-            execargs[i] = execargs[i+1];
-        }
-        execargs[i] = nullptr;
-        // do actual exec
-        setsid();
-        execvp_helper(execargs);
-        exit(0);
+    string msg = spawnProcess(input.toVector());
+    if (!msg.empty()) {
+        output.perror() << msg << endl;
+        return HERBST_INVALID_ARGUMENT;
     }
     return 0;
 }
 
-int wmexec(int argc, char** argv) {
-    if (argc >= 2) {
-        // shift all args in argv by 1 to the front
-        // so that we have space for a NULL entry at the end for execvp
-        char** execargs = argv_duplicate(argc, argv);
-        free(execargs[0]);
-        int i;
-        for (i = 0; i < argc-1; i++) {
-            execargs[i] = execargs[i+1];
-        }
-        execargs[i] = nullptr;
-        // quit and exec to new window manger
-        g_exec_args = execargs;
-    } else {
-        // exec into same command
-        g_exec_args = nullptr;
-    }
+int wmexec(Input input) {
+    g_exec_args = input.toVector();
     g_exec_before_quit = true;
     g_main_loop->quit();
     return EXIT_SUCCESS;
 }
 
-void execute_autostart_file() {
-    string path;
-    if (g_autostart_path) {
-        path = g_autostart_path;
-    } else {
-        // find right directory
-        char* xdg_config_home = getenv("XDG_CONFIG_HOME");
-        if (xdg_config_home) {
-            path = xdg_config_home;
-        } else {
-            char* home = getenv("HOME");
-            if (!home) {
-                HSWarning("Will not run autostart file. "
-                          "Neither $HOME or $XDG_CONFIG_HOME is set.\n");
-                return;
-            }
-            path = string(home) + "/.config";
-        }
-        path += "/" HERBSTLUFT_AUTOSTART;
-    }
-    if (0 == fork()) {
-        if (g_display) {
-            close(ConnectionNumber(g_display));
-        }
-        setsid();
-        execl(path.c_str(), path.c_str(), nullptr);
-
-        const char* global_autostart = HERBSTLUFT_GLOBAL_AUTOSTART;
-        HSDebug("Cannot execute %s, falling back to %s\n", path.c_str(), global_autostart);
-        execl(global_autostart, global_autostart, nullptr);
-
-        fprintf(stderr, "herbstluftwm: execvp \"%s\"", global_autostart);
-        perror(" failed");
-        exit(EXIT_FAILURE);
-    }
-}
-
 static void parse_arguments(int argc, char** argv, Globals& g) {
     int exit_on_xerror = g.exitOnXlibError;
     int noTransparency = !g.trueTransparency;
@@ -390,7 +317,7 @@ static void parse_arguments(int argc, char** argv, Globals& g) {
                     exit(0);
                 }
             case 'c':
-                g_autostart_path = optarg;
+                g.autostartPath = optarg;
                 break;
             case 'l':
                 g.initial_monitors_locked = 1;
@@ -430,13 +357,6 @@ static void parse_arguments(int argc, char** argv, Globals& g) {
     g.trueTransparency = !noTransparency;
 }
 
-static void remove_zombies(int) {
-    int bgstatus;
-    while (waitpid(-1, &bgstatus, WNOHANG) > 0) {
-        ;
-    }
-}
-
 static void handle_signal(int signal) {
     HSDebug("Interrupted by signal %d\n", signal);
     if (g_main_loop) {
@@ -459,6 +379,7 @@ static void sigaction_signal(int signum, void (*handler)(int)) {
 int main(int argc, char* argv[]) {
     Globals g;
     parse_arguments(argc, argv, g);
+    g.globalAutostartPath = HERBSTLUFT_GLOBAL_AUTOSTART;
 
     if (!setlocale(LC_CTYPE, "") || !XSupportsLocale()) {
         std::cerr << "warning: no locale support" << endl;
@@ -480,8 +401,9 @@ int main(int argc, char* argv[]) {
     if (g.trueTransparency) {
         X->tryInitTransparency();
     }
-    // remove zombies on SIGCHLD
-    sigaction_signal(SIGCHLD, remove_zombies);
+    // ignore sigchld, but set it anyway such that we don't
+    // receive anything on stopped children (SA_NOCLDSTOP)
+    sigaction_signal(SIGCHLD, [](int) {});
     sigaction_signal(SIGINT,  handle_signal);
     sigaction_signal(SIGQUIT, handle_signal);
     sigaction_signal(SIGTERM, handle_signal);
@@ -514,7 +436,12 @@ int main(int argc, char* argv[]) {
     tag_force_update_flags();
     all_monitors_apply_layout();
     ewmh->updateAll();
-    execute_autostart_file();
+    if (!root->clients->focus()) {
+        // if no client is focused, focus dummy window:
+        ewmh->clearInputFocus();
+    }
+    mainloop.childExited.connect(root->autostart(), &Autostart::childExited);
+    root->autostart()->reloadCmd();
 
     // main loop
     mainloop.run();
@@ -532,14 +459,16 @@ int main(int argc, char* argv[]) {
     delete X;
     // check if we shall restart an other window manager
     if (g_exec_before_quit) {
-        if (g_exec_args) {
+        if (!g_exec_args.empty()) {
             // do actual exec
-            HSDebug("==> Doing wmexec to %s\n", g_exec_args[0]);
-            execvp_helper(g_exec_args);
+            HSDebug("==> Doing wmexec to %s\n", g_exec_args[0].c_str());
+            int error = execvp_helper(g_exec_args);
+            std::cerr << argv[0] << ": execvp \"" << g_exec_args[0]
+                      << "\" failed: " << trimRight(strerror(error), "\n") << endl;
         }
         // on failure or if no other wm given, then fall back
         HSDebug("==> Doing wmexec to %s\n", argv[0]);
-        execvp_helper(argv);
+        execvp(argv[0], argv);
         return EXIT_FAILURE;
     }
     return EXIT_SUCCESS;
