@@ -12,6 +12,7 @@
 #include "clientmanager.h"
 #include "command.h"
 #include "completion.h"
+#include "decoration.h"
 #include "frametree.h"
 #include "globals.h"
 #include "ipc-protocol.h"
@@ -21,8 +22,11 @@
 #include "mousedraghandler.h"
 #include "root.h"
 #include "tag.h"
+#include "x11-utils.h"
 
+using std::make_shared;
 using std::vector;
+using std::shared_ptr;
 using std::string;
 using std::endl;
 
@@ -30,6 +34,7 @@ MouseManager::MouseManager()
     : dragHandler_({})
     , clients_(nullptr)
     , monitors_(nullptr)
+    , tags_(nullptr)
 {
     /* set cursor theme */
     cursor = XCreateFontCursor(g_display, XC_left_ptr);
@@ -47,10 +52,11 @@ MouseManager::~MouseManager() {
     XFreeCursor(g_display, cursor);
 }
 
-void MouseManager::injectDependencies(ClientManager* clients, MonitorManager* monitors)
+void MouseManager::injectDependencies(ClientManager* clients, TagManager* tags, MonitorManager* monitors)
 {
     clients_ = clients;
     monitors_ = monitors;
+    tags_ = tags;
 }
 
 int MouseManager::addMouseBindCommand(Input input, Output output) {
@@ -169,31 +175,51 @@ string MouseManager::mouse_initiate_move(Client* client, const vector<string> &c
     return mouse_initiate_drag(
                 client,
                 MouseDragHandlerFloating::construct(
-                    &MouseDragHandlerFloating::mouse_function_move));
+                    &MouseDragHandlerFloating::mouse_function_move),
+                    {});
 }
 
 string MouseManager::mouse_initiate_zoom(Client* client, const vector<string> &cmd) {
     MouseDragHandler::Constructor constructor;
+    ResizeAction ra = client->dec->resizeFromRoughCursorPosition(get_cursor_position());
     if (client->is_client_floated()) {
         constructor = MouseDragHandlerFloating::construct(
                          &MouseDragHandlerFloating::mouse_function_zoom);
     } else {
         auto frame = client->tag()->frame->findFrameWithClient(client);
-        constructor = MouseResizeFrame::construct(frame);
+        constructor = MouseResizeFrame::construct(frame, ra);
     }
-    return mouse_initiate_drag(client, constructor);
+    return mouse_initiate_drag(client, constructor, ra);
 }
 
 string MouseManager::mouse_initiate_resize(Client* client, const vector<string> &cmd) {
     MouseDragHandler::Constructor constructor;
+    ResizeAction ra = client->dec->resizeFromRoughCursorPosition(get_cursor_position());
     if (client->is_client_floated()) {
         constructor = MouseDragHandlerFloating::construct(
                          &MouseDragHandlerFloating::mouse_function_resize);
     } else {
         auto frame = client->tag()->frame->findFrameWithClient(client);
-        constructor = MouseResizeFrame::construct(frame);
+        constructor = MouseResizeFrame::construct(frame, ra);
     }
-    return mouse_initiate_drag(client, constructor);
+    return mouse_initiate_drag(client, constructor, ra);
+}
+
+string MouseManager::mouse_initiate_resize(Client* client, const ResizeAction& resize)
+{
+    MouseDragHandler::Constructor constructor;
+    if (client->is_client_floated()) {
+        constructor = [resize](MonitorManager* monitors, TagManager* tags, Client* clientInner) -> shared_ptr<MouseDragHandler> {
+            auto mdh = make_shared<MouseDragHandlerFloating>(monitors, tags, clientInner, &MouseDragHandlerFloating::mouse_function_resize);
+            mdh->lockWidth = !resize.left && !resize.right;
+            mdh->lockHeight = !resize.top && !resize.bottom;
+            return mdh;
+        };
+    } else {
+        auto frame = client->tag()->frame->findFrameWithClient(client);
+        constructor = MouseResizeFrame::construct(frame, resize);
+    }
+    return mouse_initiate_drag(client, constructor, resize);
 }
 
 string MouseManager::mouse_call_command(Client* client, const vector<string> &cmd) {
@@ -214,10 +240,24 @@ string MouseManager::mouse_call_command(Client* client, const vector<string> &cm
     return {};
 }
 
-string MouseManager::mouse_initiate_drag(Client *client, const MouseDragHandler::Constructor& createHandler)
+/**
+ * @brief If a client is dragged, provide information how the drag
+ * affects the client's geometry.
+ * @return
+ */
+ResizeAction MouseManager::resizeAction()
+{
+    if (dragHandler_) {
+        return dragHandler_->resizeAction_;
+    }
+    return {};
+}
+
+string MouseManager::mouse_initiate_drag(Client *client, const MouseDragHandler::Constructor& createHandler, ResizeAction resize)
 {
     try {
-        dragHandler_ = createHandler(monitors_, client);
+        dragHandler_ = createHandler(monitors_, tags_, client);
+        dragHandler_->resizeAction_ = resize * client->possibleResizeActions();
         // only grab pointer if dragHandler_ could be started
         clients_->setDragged(client);
     }  catch (const MouseDragHandler::DragNotPossible& e) {
