@@ -56,6 +56,7 @@ public:
     Attribute_<bool>          raise_on_focus_temporarily = {"raise_on_focus_temporarily", false};
     Attribute_<bool>          raise_on_click = {"raise_on_click", true};
     Attribute_<bool>          gapless_grid = {"gapless_grid", true};
+    Attribute_<bool>          tabbed_max = {"tabbed_max", true};
     Attribute_<bool>          hide_covered_windows = {"hide_covered_windows", false};
     Attribute_<bool>          smart_frame_surroundings = {"smart_frame_surroundings", false};
     Attribute_<bool>          smart_window_surroundings = {"smart_window_surroundings", false};
@@ -64,6 +65,7 @@ public:
     Attribute_<bool>          auto_detect_panels = {"auto_detect_panels", true};
     Attribute_<int>           pseudotile_center_threshold = {"pseudotile_center_threshold", 10};
     Attribute_<bool>          update_dragged_clients = {"update_dragged_clients", false};
+    Attribute_<string>        ellipsis = {"ellipsis", "..."};
     Attribute_<string>        tree_style = {"tree_style", "*| +`--."};
     Attribute_<string>        wmname = {"wmname", WINDOW_MANAGER_NAME};
     // for compatibility
