@@ -3,6 +3,8 @@
 #include <X11/X.h>
 #include <X11/Xatom.h>
 #include <X11/Xlib.h>
+#include <X11/cursorfont.h>
+#include <sys/wait.h>
 #include <iostream>
 #include <memory>
 
@@ -31,6 +33,7 @@
 #include "watchers.h"
 #include "xconnection.h"
 
+using std::make_pair;
 using std::function;
 using std::shared_ptr;
 using std::string;
@@ -80,6 +83,23 @@ XMainLoop::XMainLoop(XConnection& X, Root* root)
 
     root_->clients->dragged.changed()
             .connect(this, &XMainLoop::draggedClientChanges);
+    root_->clients->focus.changed()
+            .connect(this, &XMainLoop::focusedClientChanges);
+
+    // detect compositor
+    X_.setCompositorRunning(root_->ewmh_.detectCompositingManager());
+    // get events if the compositor changes
+    if (XFixesQueryExtension (X_.display(), &xfixesEventBase_, &xfixesErrorBase_)) {
+        int mask =  XFixesSetSelectionOwnerNotifyMask
+                    | XFixesSelectionWindowDestroyNotifyMask
+                    | XFixesSelectionClientCloseNotifyMask;
+        XFixesSelectSelectionInput(X_.display(),
+                                   X_.root(),
+                                   root->ewmh_.compositingManagerSelection(),
+                                   mask);
+    } else {
+        HSDebug("XFixes extension missing, so I can not react to compositors coming or leaving\n");
+    }
 }
 
 //! scan for windows and add them to the list of managed clients
@@ -165,19 +185,31 @@ void XMainLoop::run() {
     fd_set in_fds;
     x11_fd = ConnectionNumber(X_.display());
     while (!aboutToQuit_) {
+        // before making the process hang in the `select` call,
+        // first collect all zombies:
+        collectZombies();
+        // set the the `select` sets:
         FD_ZERO(&in_fds);
         FD_SET(x11_fd, &in_fds);
         // wait for an event or a signal
         select(x11_fd + 1, &in_fds, nullptr, nullptr, nullptr);
+        // if `select` was interrupted by a signal, then it was maybe SIGCHLD
+        collectZombies();
         if (aboutToQuit_) {
             break;
         }
         XSync(X_.display(), False);
         while (XQLength(X_.display())) {
             XNextEvent(X_.display(), &event);
-            EventHandler handler = handlerTable_[event.type];
-            if (handler != nullptr) {
-                (this ->* handler)(&event);
+            if (event.type < LASTEvent) {
+                EventHandler handler = handlerTable_[event.type];
+                if (handler != nullptr) {
+                    (this ->* handler)(&event);
+                }
+            } else {
+                if (event.type == xfixesEventBase_ + XFixesSelectionNotify) {
+                    selectionnotify((XFixesSelectionNotifyEvent*)&event);
+                }
             }
             root_->watchers->scanForChanges();
             XSync(X_.display(), False);
@@ -185,6 +217,19 @@ void XMainLoop::run() {
     }
 }
 
+void XMainLoop::collectZombies()
+{
+    int childInfo;
+    pid_t childPid;
+    while (true) {
+        childPid = waitpid(-1, &childInfo, WNOHANG);
+        if (childPid <= 0) {
+            break;
+        }
+        childExited.emit(make_pair(childPid, WEXITSTATUS(childInfo)));
+    }
+}
+
 void XMainLoop::quit() {
     aboutToQuit_ = true;
 }
@@ -218,13 +263,36 @@ void XMainLoop::buttonpress(XButtonEvent* be) {
             client = Decoration::toClient(be->window);
         }
         if (client) {
-            bool raise = root_->settings->raise_on_click();
-            focus_client(client, false, true, raise);
-            if (be->window == client->decorationWindow()) {
-                if (client->dec->positionTriggersResize({be->x, be->y})) {
-                    mm->mouse_initiate_resize(client, {});
+            ResizeAction resize = {};
+            bool decorationClicked = be->window == client->decorationWindow();
+            if (decorationClicked) {
+                resize = client->dec->positionTriggersResize({be->x, be->y});
+                if (resize) {
+                    resize = resize * client->possibleResizeActions();
+                }
+            }
+            if (resize) {
+                mm->mouse_initiate_resize(client, resize);
+            } else {
+                // if no resize action is triggered, check whether
+                // a tab was clicked:
+                Client* tabClient = {};
+                if (be->window == client->dec->decorationWindow()
+                    && be->button == Button1)
+                {
+                    auto maybeClick = client->dec->positionHasButton({be->x, be->y});
+                    if (maybeClick.has_value()) {
+                        tabClient = maybeClick.value().tabClient_;
+                    }
+                }
+                bool raise = root_->settings->raise_on_click();
+                if (tabClient) {
+                    focus_client(tabClient, false, true, raise);
                 } else {
-                    mm->mouse_initiate_move(client, {});
+                    focus_client(client, false, true, raise);
+                    if (decorationClicked) {
+                        mm->mouse_initiate_move(client, {});
+                    }
                 }
             }
         }
@@ -382,12 +450,16 @@ void XMainLoop::enternotify(XCrossingEvent* ce) {
     }
     // Warning: we have to set this to false again!
     duringEnterNotify_ = true;
+    Client* decorationClient = Decoration::toClient(ce->window);
+    if (decorationClient) {
+        decorationClient->dec->updateResizeAreaCursors();
+    }
     if (!root_->mouse->mouse_is_dragging()
         && root_->settings()->focus_follows_mouse()
         && ce->focus == false) {
         Client* c = root_->clients->client(ce->window);
         if (!c) {
-            c = Decoration::toClient(ce->window);
+            c = decorationClient;
         }
         shared_ptr<FrameLeaf> target;
         if (c && c->tag()->floating == false
@@ -420,8 +492,44 @@ void XMainLoop::expose(XEvent* event) {
     //HSDebug("name is: Expose for window %lx\n", ewin);
 }
 
-void XMainLoop::focusin(XEvent* event) {
-    //HSDebug("name is: FocusIn\n");
+void XMainLoop::focusin(XFocusChangeEvent* event) {
+    // get the newest FocusIn event, otherwise we could trigger
+    // an endless loop of FocusIn events
+    while (XCheckMaskEvent(X_.display(), FocusChangeMask, (XEvent *)event)) {
+        ;
+    }
+    duringFocusIn_ = true;
+    HSDebug("FocusIn for 0x%lx (%s)\n",
+            event->window,
+            XConnection::focusChangedDetailToString(event->detail));
+    if (event->type == FocusIn
+        && (event->detail == NotifyNonlinear
+            || event->detail == NotifyNonlinearVirtual))
+    {
+        Client* target = root_->clients->client(event->window);
+        if (!target) {
+            return;
+        }
+        // an event if an application steals input focus
+        // directly via XSetInputFocus, e.g. via `xdotool windowfocus`.
+        // also other applications do this, e.g. `emacsclient -n FILENAME`
+        // when an emacs window exist. There are still subtle differences between
+        // xdotool and emacsclient: xdotool generates detail=NotifyNonlinear
+        // whereas emacsclient only detail=NotifyNonlinearVirtual.
+        // I don't know how to prevent the keyboard input steal, so all we can
+        // do is to update clients.focus accordingly.
+        Window currentFocus = 0;
+        if (root_->clients->focus()) {
+            currentFocus = root_->clients->focus()->x11Window();
+        }
+        if (event->window != currentFocus) {
+            HSDebug("Window 0x%lx steals the focus\n", event->window);
+            // focus_client() does not call setInputFocus()  because
+            // duringFocusIn_ is set!
+            focus_client(target, false, true, false);
+        }
+    }
+    duringFocusIn_ = false;
 }
 
 void XMainLoop::keypress(XKeyEvent* event) {
@@ -454,11 +562,12 @@ void XMainLoop::mapnotify(XMapEvent* event) {
         // reset focus. so a new window gets the focus if it shall have the
         // input focus
         if (c == root_->clients->focus()) {
-            XSetInputFocus(X_.display(), c->window_, RevertToPointerRoot, CurrentTime);
+            setInputFocus(c);
         }
         // also update the window title - just to be sure
         c->update_title();
     } else if (!root_->ewmh_.isOwnWindow(event->window)
+               && !Decoration::toClient(event->window)
                && !is_herbstluft_window(X_.display(), event->window)) {
         // the window is not managed.
         HSDebug("MapNotify: briefly managing 0x%lx to apply rules\n", event->window);
@@ -520,6 +629,15 @@ void XMainLoop::selectionclear(XSelectionClearEvent* event)
     }
 }
 
+void XMainLoop::selectionnotify(XFixesSelectionNotifyEvent* event)
+{
+    if (event->selection == root_->ewmh_.compositingManagerSelection()) {
+        // check again, whether compositor is (still/again) running
+        X_.setCompositorRunning(root_->ewmh_.detectCompositingManager());
+        root_->monitors->relayoutAll();
+    }
+}
+
 void XMainLoop::propertynotify(XPropertyEvent* ev) {
     // printf("name is: PropertyNotify\n");
     Client* client = root_->clients->client(ev->window);
@@ -534,9 +652,13 @@ void XMainLoop::propertynotify(XPropertyEvent* ev) {
             //        ev->atom,
             //        atomname);
             if (ev->atom == XA_WM_HINTS) {
-                client->update_wm_hints();
+                bool forceNotUrgent = root_->clients->focus() == client;
+                client->readWmHints(forceNotUrgent);
             } else if (ev->atom == XA_WM_NORMAL_HINTS) {
                 client->updatesizehints();
+                Rectangle geom = client->float_size_;
+                client->applysizehints(&geom.width, &geom.height, true);
+                client->float_size_ = geom;
                 Monitor* m = find_monitor_with_tag(client->tag());
                 if (m) {
                     m->applyLayout();
@@ -560,8 +682,12 @@ void XMainLoop::propertynotify(XPropertyEvent* ev) {
 }
 
 void XMainLoop::unmapnotify(XUnmapEvent* event) {
-    HSDebug("name is: UnmapNotify for %lx\n", event->window);
-    root_->clients()->unmap_notify(event->window);
+    HSDebug("name is: UnmapNotify for window=0x%lx and event=0x%lx\n", event->window, event->event);
+    if (event->window == event->event) {
+        // reparenting the window creates multiple unmap notify events,
+        // both for the root window and the window itself.
+        root_->clients()->unmap_notify(event->window);
+    }
     if (event->send_event) {
         // if the event was synthetic, then we need to understand it as a kind request
         // by the window to be unmanaged. I don't understand fully how this is implied
@@ -582,6 +708,50 @@ void XMainLoop::unmapnotify(XUnmapEvent* event) {
     }
 }
 
+/**
+ * @brief This function is called whenever clients.focus changes
+ * @param newFocus
+ */
+void XMainLoop::focusedClientChanges(Client* newFocus)
+{
+    if (lastFocus_) {
+        root_->mouse->grab_client_buttons(lastFocus_, false);
+    }
+    if (newFocus) { // if another client is focused
+        if (!duringFocusIn_) {
+            // if the focus changes due to a FocusIn event,
+            // we do not set the input focus, because it already
+            // changed.
+            setInputFocus(newFocus);
+        }
+        root_->ewmh_.updateActiveWindow(newFocus->window_);
+        root_->mouse->grab_client_buttons(newFocus, true);
+        root_->keys()->ensureKeyMask(newFocus);
+    } else { // if no client is focused
+        Ewmh::get().clearInputFocus();
+        if (lastFocus_) {
+            root_->ewmh_.updateActiveWindow(None);
+
+            // Enable all keys in the root window
+            root_->keys()->clearActiveKeyMask();
+        }
+    }
+    lastFocus_ = newFocus;
+}
+
+/**
+ * @brief Either call XSetInputFocus directly or send WM_TAKE_FOCUS
+ * @param newFocus the client to focus
+ */
+void XMainLoop::setInputFocus(Client* newFocus)
+{
+    if (!newFocus->neverfocus_) {
+        XSetInputFocus(X_.display(), newFocus->window_, RevertToPointerRoot, CurrentTime);
+    } else {
+        root_->ewmh_.sendEvent(newFocus->window_, Ewmh::WM::TakeFocus, True);
+    }
+}
+
 IpcServer::CallResult XMainLoop::callCommand(const vector<string>& call)
 {
     // the call consists of the command and its arguments
@@ -603,10 +773,13 @@ IpcServer::CallResult XMainLoop::callCommand(const vector<string>& call)
 void XMainLoop::draggedClientChanges(Client* draggedClient)
 {
     if (draggedClient) {
+        ResizeAction ra = root_->mouse->resizeAction();
+        auto maybeCursor = ra.toCursorShape();
+        Cursor cursorShape = XCreateFontCursor(X_.display(), maybeCursor.value_or(XC_fleur));
         // listen for mouse motion events:
         XGrabPointer(X_.display(), draggedClient->x11Window(), True,
             PointerMotionMask|ButtonReleaseMask, GrabModeAsync,
-                GrabModeAsync, None, None, CurrentTime);
+                GrabModeAsync, None, cursorShape, CurrentTime);
     } else { // no client is dragged -> ungrab and clear the queue
         XUngrabPointer(X_.display(), CurrentTime);
         // remove all enternotify-events from the event queue that were
