@@ -9,15 +9,18 @@ import re
 import select
 import selectors
 import shlex
+import shutil
 import subprocess
 import sys
 import textwrap
 import time
 import types
-import herbstluftwm
 
 import pytest
 
+pytest.register_assert_rewrite("herbstluftwm")
+import herbstluftwm  # noqa: E402
+
 
 BINDIR = os.path.join(os.path.abspath(os.environ['PWD']))
 
@@ -26,7 +29,7 @@ BINDIR = os.path.join(os.path.abspath(os.environ['PWD']))
 COPY_ENV_WHITELIST = ['LSAN_OPTIONS']
 
 # time in seconds to wait for a process to shut down
-PROCESS_SHUTDOWN_TIME = 10
+PROCESS_SHUTDOWN_TIME = 30
 
 
 def extend_env_with_whitelist(environment):
@@ -130,7 +133,7 @@ class HlwmBridge(herbstluftwm.Herbstluftwm):
         attribute_path = '.'.join([str(x) for x in attribute_path])
         return self.call(['get_attr', attribute_path]).stdout
 
-    def create_client(self, term_command='sleep infinity', position=None, title=None, keep_running=False):
+    def create_client(self, term_command='while sleep 3600; do true; done', position=None, title=None, keep_running=False):
         """
         Launch a client that will be terminated on shutdown.
         """
@@ -141,8 +144,11 @@ class HlwmBridge(herbstluftwm.Herbstluftwm):
         if position is not None:
             x, y = position
             geometry[1] = '50x2%+d%+d' % (x, y)
-        command = ['xterm'] + title + geometry
-        command += ['-class', wmclass, '-e', 'bash', '-c', term_command]
+        # shutil.which is necessary, because we reset the environment and
+        # therefore PATH.
+        # https://docs.python.org/3/library/subprocess.html#subprocess.Popen
+        command = [shutil.which('xterm')] + title + geometry
+        command += ['-class', wmclass, '-e', shutil.which('bash'), '-c', term_command]
 
         # enforce a hook when the window appears
         self.call(['rule', 'once', 'class=' + wmclass, 'hook=here_is_' + wmclass])
@@ -267,6 +273,11 @@ class HlwmBridge(herbstluftwm.Herbstluftwm):
         for client_proc in self.client_procs:
             client_proc.wait(PROCESS_SHUTDOWN_TIME)
         self.hc_idle.wait(PROCESS_SHUTDOWN_TIME)
+        self.hc_idle.stdout.close()
+
+        # test that the client itself is still working:
+        assert self.call(['echo', 'ping before shutdown']).stdout \
+            == 'ping before shutdown\n'
 
     def bool(self, python_bool_var):
         """convert a boolean variable into hlwm's string representation"""
@@ -448,6 +459,8 @@ class HlwmProcess:
 
         # Make sure to read and echo all remaining output (esp. ASAN messages):
         self.read_and_echo_output(until_eof=True)
+        self.proc.stdout.close()
+        self.proc.stderr.close()
 
         if self.proc.returncode is None:
             # only wait the process if it hasn't been cleaned up
@@ -524,6 +537,7 @@ class HcIdle:
         except subprocess.TimeoutExpired:
             self.proc.kill()
             self.proc.wait(PROCESS_SHUTDOWN_TIME)
+        self.proc.stdout.close()
 
 
 @pytest.fixture()
@@ -677,6 +691,13 @@ class RawImage:
                 count += 1
         return count
 
+    def color_count_dict(self):
+        """return a dictionary telling how often each color occurs"""
+        result = {}
+        for pix in self.data:
+            result[pix] = result.get(pix, 0) + 1
+        return result
+
     @staticmethod
     def rgb2string(rgb_triple):
         return '#%02x%02x%02x' % rgb_triple
@@ -699,13 +720,17 @@ class X11:
     def winid_str(self, window_handle):
         return hex(window_handle.id)
 
-    def make_window_urgent(self, window):
+    def make_window_urgent(self, window, sync_hlwm=True):
         """make window urgent"""
         window.set_wm_hints(flags=Xutil.UrgencyHint)
         self.display.sync()
+        if sync_hlwm:
+            # wait for hlwm to fully recognize the new wm hint
+            self.sync_with_hlwm()
 
     def is_window_urgent(self, window):
         """check urgency of a given window handle"""
+        self.display.sync()
         hints = window.get_wm_hints()
         if hints is None:
             return False
@@ -752,6 +777,7 @@ class X11:
                       window_type=None,
                       transient_for=None,
                       pre_map=lambda wh: None,  # called before the window is mapped
+                      pre_sync=lambda wh: None,  # called before the display is synced
                       ):
         w = self.root.create_window(
             geometry[0],
@@ -792,6 +818,7 @@ class X11:
 
         pre_map(w)
         w.map()
+        pre_sync(w)
         self.display.sync()
         if sync_hlwm:
             # wait for hlwm to fully recognize it as a client
@@ -847,6 +874,10 @@ class X11:
         decoration = self.get_decoration_window(win_handle)
         return self.screenshot(decoration)
 
+    def set_window_title(self, win_handle, title):
+        self.set_property_textlist('_NET_WM_NAME', [title], window=win_handle)
+        self.sync_with_hlwm()
+
     def sync_with_hlwm(self):
         self.display.sync()
         # wait for hlwm to flush all events:
@@ -861,6 +892,15 @@ class X11:
         else:
             return tree.parent
 
+    def get_window_under_cursor(self):
+        parent = None
+        window = self.root
+        while window is not None and window != 0:
+            parent = window
+            window = window.query_pointer().child
+            print("window under cursor: {}".format(window))
+        return parent
+
     def get_absolute_top_left(self, window):
         """return the absolute (x,y) coordinate of the given window,
         i.e. relative to the root window"""
@@ -912,6 +952,18 @@ class X11:
             window.destroy()
         self.display.sync()
 
+    def pop_pending_events(self):
+        """extract all pending events from the event queue"""
+        events = []
+        while True:
+            self.display.sync()
+            count = self.display.pending_events()
+            if count <= 0:
+                break
+            for i in range(0, count):
+                events.append(self.display.next_event())
+        return events
+
 
 @pytest.fixture()
 def x11(x11_connection):
@@ -987,7 +1039,11 @@ class MultiscreenDisplay:
 
     def __exit__(self, type_param, value, traceback):
         self.proc.terminate()
-        self.proc.wait(PROCESS_SHUTDOWN_TIME)
+        try:
+            self.proc.wait(PROCESS_SHUTDOWN_TIME)
+        except subprocess.TimeoutExpired:
+            self.proc.kill()
+            self.proc.wait(PROCESS_SHUTDOWN_TIME)
 
 
 @pytest.fixture()
@@ -1045,7 +1101,7 @@ def mouse(hlwm_process, hlwm):
                 hlwm.call('true')
 
         def move_relative(self, delta_x, delta_y, wait=True):
-            self.call_cmd(f'xdotool mousemove_relative --sync {delta_x} {delta_y}', shell=True)
+            self.call_cmd(f'xdotool mousemove_relative --sync -- {delta_x} {delta_y}', shell=True)
             if wait:
                 # wait until all the mouse move events that were put in the
                 # queue by the above xdotool invokation are fully processed.
