@@ -112,9 +112,13 @@ def types_and_shorthands():
         'regex': 'r',
         'SplitAlign': 'n',
         'LayoutAlgorithm': 'n',
+        'TextAlign': 'n',
+        'TitleWhen': 'n',
         'font': 'f',
         'Rectangle': 'R',
         'WindowID': 'w',
+        'MaybeColor': 's',
+        'MaybeULong': 's',
     }
 
 
@@ -211,3 +215,32 @@ def test_help_on_children_vs_json(hlwm, clsname, object_path, json_doc):
             # then there should be one in the cpp source
             # and thus also not in the help output:
             assert f"Entry '{name}'" not in help_txt
+
+
+@pytest.mark.parametrize('clsname,object_path', classname2examplepath)
+def test_attribute_doc_in_json(hlwm, clsname, object_path, json_doc):
+    """
+    Verify that if an attribute has a doc string (according to the 'help' command)
+    then the doc string is also present in the json.
+    """
+    path = object_path(hlwm)
+
+    for _, attribute in json_doc['objects'][clsname]['attributes'].items():
+        name = attribute['name']
+        help_txt = hlwm.call(['help', f'{path}.{name}'.lstrip('.')]).stdout
+        help_lines = help_txt.rstrip().split('\n')
+        doc_in_help = ''
+        # the doc is everything after 'Current value: ..'
+        for line_idx in range(0, len(help_lines) - 1):
+            # a line starting with 'Current value: '
+            found = help_lines[line_idx].startswith('Current value: ')
+            # and the next line is empty:
+            found = found and help_lines[line_idx + 1] == ''
+            if found:
+                # the doc is everything after the empty line:
+                doc_in_help = '\n'.join(help_lines[line_idx + 2:])
+                break
+        if not doc_in_help.startswith('Current value:'):
+            # if there is a doc printed by 'help', then it
+            # should also be present in the json:
+            assert doc_in_help == attribute.get('doc', '').rstrip()
